 (* Teufelskurve        :                                                 *)
 (* Version 2:                                                            *)
 
 (* Unterschied zur Version1 ist:                                         *)
 (* Lineare Interpolation hilft genaue Schnittpunktbestimmung zu ermitteln*)                  
 (* genaue Vorzeichendiskussion ist notwendig !!                          *)
 (* Es werden zwei Durchgaenge gerechnet. Beim ersten Durchgang werden    *)
 (* fuer zwei benachbarte, nur durch die Schrittweite unterschiedliche    *)
 (* Werte XA und XB der dazugehoerige Funktionswert ZA und ZB berechnet   *)
 (* Der Wert fuer Y ist ja in der inneren Schleife konstant und wird in   *)
 (* der aeusseren Schleife schrittweise erhoeht. Beim zweiten Durchgang   *)
 (* wird ZA und ZB fuer zwei benachbarte Y-Werte YA und YB berechnet.     *)
 (* Ob ein Punkt in der Epsilon Umgebung liegt, wird nicht direkt durch   *)
 (* eine Wertabfrage des Absolutbetrages von Z bewerkstelligt, sondern    *)
 (* durch eine Diskussion der Vorzeichen zweier benachbarter Werte ZA,ZB  *)
 (* entschieden; mittels der Signumfunktion, welche als Unterprogramm     *)
 (* implementiert ist.                                                    *)
 (* Zwei Durchgaenge sind notwendig, um das Punktenetz dichter zu         *)
 (* legen. Nach dem 1.ten Durchgang bleiben Luecken bei den Werten, bei   *)
 (* denen die Funktion parallel zur X-Achse liegt. Beim 2.ten Durchgang   *)
 (* bleiben Luecken, wo die Funktion parallel zur Y-Achse verlaeuft       *)
 (* Dieser Bereich wurde jedoch schon sehr genau im ersten Durchgang      *)
 (* abgedeckt. Durchgaenge in Kommentarklammern setzen und ausprobieren!  *)
 (* Es wird ja nicht wie bei der Parameterdarstellung in gleichen         *)
 (* Abstaenden gepunktet.                                                 *)
(* TEUFELSKURVE -x^4 + y^4 + 50x^2 -48y^2                                 *)
(* gegenueber der zweiten Version waehlt man die Punktaufloesung          *)
(* in X-und Y-Richtung NX,NY. Die Schrittweiten DX,DY werden vom Programm *)
(* berechnet. Die Schleifenzaehler sind ganzzahlig formuliert             *)
(* das Zeichnen erfolgt in einer eigenen Prozedur                         *)
(* Um Kurvenscharen zu erzeugen dient der Parameter ZN                    *)

PROGRAM Teufel;                                              (*  1.2.1997 *)

USES CRT,UNIGRA12;

VAR
  P    : Pt2D;
  PltFileName: STRING[30];
  einh       : EXTENDED;
  NX,NY    : INTEGER;  (* Punkteanzahl in X bezw. Y Richtung              *)
  DX,DY,ZN : EXTENDED; (* DX, DY errechnete Schrittweiten                 *)
 (* ----------------------- Unterprogramme ------------------------------- *)
  Procedure Ausgangszustand_definieren;
   BEGIN
   Plotter_einschalten;
   Graphik_einschalten('');
   Weltgrenzen_festlegen(einh);  
   Viewport_festlegen;
   GeraetefensterPlotter_festlegen;
   Parameter2D;
   Viewport_zeichnen; 
   Koordinatenachsen_zeichnen;
   Gitter(0.1,1.0,5.0);  (* Strich,              klE,             grE     *)
                       (* grosse Strichlaenge,kleine Einheit, grosse Einh *)
  END;

  Procedure Beschriftung;
  CONST
  N = 10;
  VAR
  XUntergr,XObergr,YUntergr,YObergr         : STRING[Bz1];
  einhS: STRING[Bz1];
  Ttemp1,Ttemp2: STRING[Bz2];
  T: Ttyp;
  PText : Pt2DInt;
  BEGIN
  STR(WI2D[1].X:5:1,XUntergr); STR(WI2D[1].Y:5:1,YUntergr);
  STR(WI2D[2].X:5:1,XObergr);  STR(WI2D[2].Y:5:1,YObergr);
  STR(einh:5:1,einhS);

  T[1] := 'Teufelskurve: -x^4 + y^4 + 50x^2 -48y^2';
 
        Ttemp1 := 'Weltgrenzen: X von '+XUntergr+' bis '+XObergr;
        Ttemp2 := '   Y von '+YUntergr+' bis '+YObergr;
  T[2] := Ttemp1 + Ttemp2;
  T[3] := '1 Einheit entspricht'+einhS+'  cm';
  Text_ausgeben(T,3);                                (* .. Texte ausgeben *)
  END;

  FUNCTION Sgn(A : EXTENDED) : INTEGER;
  CONST
  EPS = 1.0E-09;
  BEGIN
  IF(ABS(A) < EPS) THEN BEGIN
      Sgn := 0;
  END ELSE BEGIN IF (A/ABS(A)) > 0 THEN BEGIN
             Sgn := 1;
           END ELSE BEGIN Sgn := -1;
               END;
           END;      
  END;

  FUNCTION ZET(X,Y:EXTENDED):EXTENDED;
  VAR
  F1 : EXTENDED;
  BEGIN
  F1 := -(X*X*X*X)+(Y*Y*Y*Y)+(50*X*X)-(48*Y*Y);
  ZET := F1 - ZN;
  END;

PROCEDURE Zeich;
VAR
I,J : INTEGER;
P,PA,PB,P0 : Pt2D;
Z,ZA,ZB : EXTENDED;
H : Pt2D;
  BEGIN
  (* ---------------- 1.ter Durchgang beginnt --------------------------- *)
  P.Y := WI2D[1].Y;
  J := 0;
  WHILE (J < NY ) DO BEGIN
     P.Y := WI2D[1].Y + J * DY;
     I := 0;
     WHILE (I < NX) DO BEGIN
         PA.X := WI2D[1].X + I*DX;
         PB.X := PA.X + DX;     (*   PB.X := WI2D[1].X + (I + 1)*DX;   *)
         (* ====================================== *)
         ZA := ZET(PA.X,P.Y);
         ZB := ZET(PB.X,P.Y);
         (* ====================================== *)
          
         IF (Sgn(ZA) * Sgn(ZB) = -1 )THEN BEGIN
            P0.X := (PA.X*ZB - PB.X*ZA)/(ZB-ZA);
            H.X := P0.X; H.Y := P.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)=0) THEN BEGIN
            P0.X := (PA.X + PB.X)/2;
            H.X := P0.X; H.Y := P.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)<>0) THEN BEGIN
            P0.X := PA.X;
            H.X := P0.X; H.Y := P.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)<>0) AND (Sgn(ZB)=0) THEN BEGIN
            P0.X := PB.X;
            H.X := P0.X; H.Y := P.Y;
            PutPt2D(H,Col);
         END;                   
         I := I + 1;
    END;
    J := J + 1;
  END;             
  (* --------------------- 2.ter Durchgang beginnt ---------------------- *)
  P.X := WI2D[1].X;
  I := 0;
  WHILE (I < NX ) DO BEGIN
     P.X := WI2D[1].X + I * DX;
     J := 0;
     WHILE (J < NY) DO BEGIN
         PA.Y := WI2D[1].Y + J*DY;
         PB.Y := PA.Y + DY;     (*   PB.Y := WI2D[1].Y + (J + 1)*DY;   *)
         (* ====================================== *)
         ZA := ZET(P.X,PA.Y);
         ZB := ZET(P.X,PB.Y);
         (* ====================================== *)
         IF Sgn(ZA) * Sgn(ZB) = -1 THEN BEGIN
            P0.Y := (PA.Y*ZB - PB.Y*ZA)/(ZB-ZA);
            H.X := P.X; H.Y := P0.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)=0) THEN BEGIN
            P0.Y := (PA.Y + PB.Y)/2;
            H.X := P.X; H.Y := P0.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)=0) AND (Sgn(ZB)<>0) THEN BEGIN
            P0.Y := PA.Y;
            H.X := P.X; H.Y := P0.Y;
            PutPt2D(H,Col);
         END;
         IF (Sgn(ZA)<>0) AND (Sgn(ZB)=0) THEN BEGIN
            P0.Y := PB.Y;
            H.X := P.X; H.Y := P0.Y;
            PutPt2D(H,Col);
         END;
         J := J + 1;
    END;
    I := I + 1;
  END;
  END;

 (* ====================================================================  *)
 (*                         HAUPTPROGRAMM                                 *)

BEGIN
  Schnittstelle_waehlen(1);               (* 2 bedeutet auf Datei plotten *)
  Plot := FALSE;  
  einh := 1.0;   (* 1 Einheit auf der X-Achse und Y-Achse entspricht 1 cm *)
  (* erster Punkt links unten kann gewaehlt werden *)
  WI2D[1].X := -14.0;  WI2D[1].Y := -10.0;    (* linker unterer Punkt     *)
                                              (* des Weltfensters         *)
  Ausgangszustand_definieren;                (* Graphikgrundeinstellung   *)
  (* NX Punktaufloesung in X-Richtung waehlen                             *)
  (* NY Punktaufloesung in Y-Richtung waehlen                             *)
  NX := 200; NY := 200;
  (* Schrittweiten werden berechnet                                       *)
  DX := (WI2D[2].X - WI2D[1].X)/NX; DY := (WI2D[2].Y - WI2D[1].Y)/NY;
  ZN := -1000.0;
  WHILE ZN <= 1000.0 DO BEGIN
     Zeich;
     ZN := ZN + 200.0;
  END;                     
  Beschriftung;
  WClosegraph;
END.
 
      